#include <cstdio>
#include <iostream>

#include <windows.h>
#include <WinSock2.h> // struct timeval
#include "msg.h"

enum LogLevel {
   kError = 1,
   kWarning = 2,
   kInfo = 3,
   kDebug = 4
};

static void do_log(int type, int id, const char *fmt, va_list ap) {

	/* Get the length of the message and allocate buffer */
	int len=_vscprintf(fmt,ap);
	char *buf=new char[len+1];

	/* Format the message */
	vsprintf(buf,fmt,ap);

	/* Log the message as above. */
	HANDLE hEventLog;
	hEventLog = RegisterEventSource(NULL, "MSG");

	ReportEvent(hEventLog,
				type,         // event type
				0,            // category
				id,           // event id
				NULL,         // security id
				1,            // one field
				0,            // no data
				(LPCTSTR *)&buf, // pointer to string array
				NULL);        // pointer to data

	DeregisterEventSource(hEventLog);

	/* Free buffer. */
	delete[] buf;
}

void LogErrorF(const char *fmt, ...)
{
	va_list ap;
	va_start(ap,fmt);
	do_log(EVENTLOG_ERROR_TYPE,MSG_ERROR,fmt,ap);
	va_end(ap);
}

void LogWarningF(const char *fmt, ...)
{
	va_list ap;
	va_start(ap,fmt);
	do_log(EVENTLOG_WARNING_TYPE,MSG_WARNING,fmt,ap);
	va_end(ap);
}

void LogInfoF(const char *fmt, ...)
{
	va_list ap;
	va_start(ap,fmt);
	do_log(EVENTLOG_INFORMATION_TYPE,MSG_INFO,fmt,ap);
	va_end(ap);
}

void LogDebugF(const char *fmt, ...)
{
	va_list ap;
	va_start(ap,fmt);
	do_log(EVENTLOG_INFORMATION_TYPE,MSG_DEBUG,fmt,ap);
	va_end(ap);
}


void LogMessage(int level, const char *msg)
{
    // prevent msg from being processed as format string
    // by passing after "%s"...
    switch (level) {
      case kError:
        LogErrorF("%s", msg);
        break;
      case kWarning:
        LogWarningF("%s", msg);
        break;
      case kInfo:
        LogInfoF("%s", msg);
        break;
      case kDebug:
        LogDebugF("%s", msg);
        break;
      default:
        0 == level ? LogErrorF("%s", msg) : LogDebugF("%s", msg); // extra debug levels
        break;
    }            
}
